/* 
 * FILE --- fuzzy_mf.cc
 *
 * PURPOSE --- Source file for a class for fuzzy membership functions (MF).
 * The MF could be chosen to be either triangular or Gaussian. Every other 
 * MF type is interpreted as triangular MF. The user specifies the center 
 * and the spread of the MF.
 */

// Include Files
#include <math.h>    /* exp function */
#include "fuzzy_mf.hh"

// Define this macro if you want to use this utility without linking
// to the RCS Library
#ifdef NO_RCS
#include <stdio.h>            /* printf function */
#else
#include "rcs_prnt.hh"          /* rcs_print_ functions */
#endif


/*************************************************************/

FUZZY_MF::FUZZY_MF()
{
  // Initialize
  set_center(0.0);
  set_spread(1.0);
  set_type(TRIANGULAR_MF);
}

FUZZY_MF::FUZZY_MF(double _center, 
		   double _spread, 
		   int _type = TRIANGULAR_MF)
{
  // Initialize
  set_center(_center);
  set_spread(_spread);
  set_type(_type);
}

void FUZZY_MF::set_mf(const FUZZY_MF &_mf)
{
  type = _mf.type;
  center = _mf.center;
  spread = _mf.spread;
}

// This function sets the membership function type. Invalid 
// types are considered as TRIANGULAR_MF.
void FUZZY_MF::set_type(int _type)
{
  type = _type;
  if((type != TRIANGULAR_MF) && (type != GAUSSIAN_MF))
    {
#ifdef NO_RCS
      printf("Invalid membership function type!!!\n");
      printf("Triangular membership function assumed ...\n");
#else
      rcs_print("Invalid membership function type!!!\n");
      rcs_print("Triangular membership function assumed ...\n");
#endif
      type = TRIANGULAR_MF;
    }
}

double FUZZY_MF::find_dom(const double _input)
{
  double output;

  switch(type)
    {
    case TRIANGULAR_MF:
      output = triangle(_input);
      break;

    case GAUSSIAN_MF:
      output = gaussian(_input);
      break;

    default:
      // This wont happen, but just in case ...
      // Play Safe --- Consider every other type as triangular
      output = triangle(_input);
    }

  return(output);
}

inline double FUZZY_MF::triangle(const double _input)
{
  return (MAX(0.0,(1.0-fabs(_input-center)/spread)));
}

inline double FUZZY_MF::gaussian(const double _input)
{
  return ( exp( -pow(((_input - center)/spread), 2.0) ) );
}

